@@ -136,6 +136,10 @@ macro_rules! tok {
        swc_css_ast::Token::Delim { value: '/', .. }
     };
 
+    ("<") => {
+        swc_css_ast::Token::Delim { value: '<', .. }
+    };
+
     (">") => {
         swc_css_ast::Token::Delim { value: '>', .. }
     };
