@@ -5,7 +5,7 @@ use crate::{
     Parse,
 };
 use swc_atoms::js_word;
-use swc_common::{Span, Spanned, DUMMY_SP};
+use swc_common::{Span, DUMMY_SP};
 use swc_css_ast::*;
 
 #[derive(Debug, Default)]
@@ -666,7 +666,7 @@ where
 {
     fn parse(&mut self) -> PResult<MediaRule> {
         let span = self.input.cur_span()?;
-        let query = self.parse()?;
+        let media = self.parse()?;
 
         expect!(self, "{");
 
@@ -678,123 +678,496 @@ where
 
         Ok(MediaRule {
             span: span!(self, span.lo),
-            query,
+            media,
             rules,
         })
     }
 }
 
+impl<I> Parse<MediaQueryList> for Parser<I>
+where
+    I: ParserInput,
+{
+    fn parse(&mut self) -> PResult<MediaQueryList> {
+        self.input.skip_ws()?;
+
+        let query = self.parse()?;
+        let mut queries = vec![query];
+
+        // TODO error recovery
+        // To parse a <media-query-list> production, parse a comma-separated list of
+        // component values, then parse each entry in the returned list as a
+        // <media-query>. Its value is the list of <media-query>s so produced.
+        loop {
+            self.input.skip_ws()?;
+
+            if !eat!(self, ",") {
+                break;
+            }
+
+            self.input.skip_ws()?;
+
+            let query = self.parse()?;
+
+            queries.push(query);
+        }
+
+        let start_pos = match queries.first() {
+            Some(MediaQuery { span, .. }) => span.lo,
+            _ => {
+                unreachable!();
+            }
+        };
+        let last_pos = match queries.last() {
+            Some(MediaQuery { span, .. }) => span.hi,
+            _ => {
+                unreachable!();
+            }
+        };
+
+        Ok(MediaQueryList {
+            span: Span::new(start_pos, last_pos, Default::default()),
+            queries,
+        })
+    }
+}
+
 impl<I> Parse<MediaQuery> for Parser<I>
 where
     I: ParserInput,
 {
     fn parse(&mut self) -> PResult<MediaQuery> {
-        self.input.skip_ws()?;
+        let start_pos = self.input.cur_span()?.lo;
+        let state = self.input.state();
 
-        let span = self.input.cur_span()?;
+        // TODO uppercase acceptable
+        let mut modifier = if is!(self, "not") || is!(self, "only") {
+            let modifier = Some(self.parse()?);
 
-        let base = if eat!(self, "not") {
-            let query = self.parse()?;
-            MediaQuery::Not(NotMediaQuery {
-                span: span!(self, span.lo),
-                query,
-            })
-        } else if eat!(self, "only") {
-            let query = self.parse()?;
-            MediaQuery::Only(OnlyMediaQuery {
-                span: span!(self, span.lo),
-                query,
-            })
-        } else if is!(self, Ident) {
-            let ident = self.parse()?;
-            MediaQuery::Ident(ident)
-        } else if eat!(self, "(") {
-            if is!(self, Ident) {
-                let span = self.input.cur_span()?;
-                let id = self.parse()?;
+            self.input.skip_ws()?;
 
-                self.input.skip_ws()?;
-
-                if eat!(self, ":") {
-                    self.input.skip_ws()?;
-
-                    let ctx = Ctx {
-                        allow_operation_in_value: true,
-                        ..self.ctx
-                    };
-                    let value = self.with_ctx(ctx).parse_property_values()?.0;
-
-                    expect!(self, ")");
-
-                    MediaQuery::Declaration(Declaration {
-                        span: span!(self, span.lo),
-                        property: id,
-                        value,
-                        important: Default::default(),
-                    })
-                } else {
-                    expect!(self, ")");
-                    MediaQuery::Ident(id)
-                }
-            } else {
-                let query: MediaQuery = self.parse()?;
-                expect!(self, ")");
-
-                query
-            }
+            modifier
         } else {
-            return Err(Error::new(span, ErrorKind::InvalidMediaQuery));
+            None
         };
 
+        let mut last_pos = self.input.last_pos()?;
+
+        let media_type = if !is!(self, Ident) || is_one_of!(self, "not", "and", "or", "only") {
+            None
+        } else {
+            let media_type = Some(self.parse()?);
+
+            last_pos = self.input.last_pos()?;
+
+            self.input.skip_ws()?;
+
+            media_type
+        };
+
+        let condition = if media_type.is_some() {
+            if eat!(self, "and") {
+                self.input.skip_ws()?;
+
+                let condition_without_or: MediaConditionWithoutOr = self.parse()?;
+
+                last_pos = condition_without_or.span.hi;
+
+                Some(MediaConditionType::WithoutOr(condition_without_or))
+            } else {
+                None
+            }
+        } else {
+            modifier = None;
+
+            self.input.reset(&state);
+
+            let condition: MediaCondition = self.parse()?;
+
+            last_pos = condition.span.hi;
+
+            Some(MediaConditionType::All(condition))
+        };
+
+        Ok(MediaQuery {
+            span: Span::new(start_pos, last_pos, Default::default()),
+            modifier,
+            media_type,
+            condition,
+        })
+    }
+}
+
+impl<I> Parse<MediaCondition> for Parser<I>
+where
+    I: ParserInput,
+{
+    fn parse(&mut self) -> PResult<MediaCondition> {
         self.input.skip_ws()?;
 
-        if eat!(self, "and") {
-            let right: Box<MediaQuery> = self.parse()?;
+        let start_pos = self.input.cur_span()?.lo;
+        let mut last_pos;
+        let mut conditions = vec![];
 
-            return Ok(MediaQuery::And(AndMediaQuery {
-                span: Span::new(span.lo, right.span().hi, Default::default()),
-                left: Box::new(base),
-                right,
-            }));
-        }
+        if is!(self, "not") {
+            let not = self.parse()?;
 
-        if eat!(self, "or") {
-            let right: Box<MediaQuery> = self.parse()?;
+            last_pos = self.input.last_pos()?;
 
-            return Ok(MediaQuery::Or(OrMediaQuery {
-                span: Span::new(span.lo, right.span().hi, Default::default()),
-                left: Box::new(base),
-                right,
-            }));
-        }
+            conditions.push(MediaConditionAllType::Not(not));
+        } else {
+            let media_in_parens = self.parse()?;
 
-        if !self.ctx.disallow_comma_in_media_query && eat!(self, ",") {
-            let mut queries = Vec::with_capacity(4);
-            queries.push(base);
+            last_pos = self.input.last_pos()?;
 
-            loop {
-                self.input.skip_ws()?;
+            conditions.push(MediaConditionAllType::MediaInParens(media_in_parens));
 
-                let ctx = Ctx {
-                    disallow_comma_in_media_query: true,
-                    ..self.ctx
-                };
-                let q = self.with_ctx(ctx).parse_with(|p| p.parse())?;
-                queries.push(q);
+            self.input.skip_ws()?;
 
-                self.input.skip_ws()?;
-                if !eat!(self, ",") {
-                    break;
+            if is!(self, "and") {
+                while is!(self, "and") {
+                    let and = self.parse()?;
+
+                    last_pos = self.input.last_pos()?;
+
+                    conditions.push(MediaConditionAllType::And(and));
+
+                    self.input.skip_ws()?;
+                }
+            } else if is!(self, "or") {
+                while is!(self, "or") {
+                    let or = self.parse()?;
+
+                    last_pos = self.input.last_pos()?;
+
+                    conditions.push(MediaConditionAllType::Or(or));
+
+                    self.input.skip_ws()?;
                 }
             }
+        };
 
-            return Ok(MediaQuery::Comma(CommaMediaQuery {
-                span: span!(self, span.lo),
-                queries,
-            }));
+        Ok(MediaCondition {
+            span: Span::new(start_pos, last_pos, Default::default()),
+            conditions,
+        })
+    }
+}
+
+impl<I> Parse<MediaConditionWithoutOr> for Parser<I>
+where
+    I: ParserInput,
+{
+    fn parse(&mut self) -> PResult<MediaConditionWithoutOr> {
+        self.input.skip_ws()?;
+
+        let start_pos = self.input.cur_span()?.lo;
+        let mut last_pos;
+        let mut conditions = vec![];
+
+        if is!(self, "not") {
+            let not = self.parse()?;
+
+            last_pos = self.input.last_pos()?;
+
+            conditions.push(MediaConditionWithoutOrType::Not(not));
+        } else {
+            let media_in_parens = self.parse()?;
+
+            last_pos = self.input.last_pos()?;
+
+            conditions.push(MediaConditionWithoutOrType::MediaInParens(media_in_parens));
+
+            self.input.skip_ws()?;
+
+            if is!(self, "and") {
+                while is!(self, "and") {
+                    let and = self.parse()?;
+
+                    last_pos = self.input.last_pos()?;
+
+                    conditions.push(MediaConditionWithoutOrType::And(and));
+
+                    self.input.skip_ws()?;
+                }
+            }
+        };
+
+        Ok(MediaConditionWithoutOr {
+            span: Span::new(start_pos, last_pos, Default::default()),
+            conditions,
+        })
+    }
+}
+
+impl<I> Parse<MediaNot> for Parser<I>
+where
+    I: ParserInput,
+{
+    fn parse(&mut self) -> PResult<MediaNot> {
+        let span = self.input.cur_span()?;
+
+        expect!(self, "not");
+
+        self.input.skip_ws()?;
+
+        let media_in_parens = self.parse()?;
+
+        Ok(MediaNot {
+            span: span!(self, span.lo),
+            condition: media_in_parens,
+        })
+    }
+}
+
+impl<I> Parse<MediaAnd> for Parser<I>
+where
+    I: ParserInput,
+{
+    fn parse(&mut self) -> PResult<MediaAnd> {
+        let span = self.input.cur_span()?;
+
+        expect!(self, "and");
+
+        self.input.skip_ws()?;
+
+        let media_in_parens = self.parse()?;
+
+        Ok(MediaAnd {
+            span: span!(self, span.lo),
+            condition: media_in_parens,
+        })
+    }
+}
+
+impl<I> Parse<MediaOr> for Parser<I>
+where
+    I: ParserInput,
+{
+    fn parse(&mut self) -> PResult<MediaOr> {
+        let span = self.input.cur_span()?;
+
+        expect!(self, "or");
+
+        self.input.skip_ws()?;
+
+        let media_in_parens = self.parse()?;
+
+        Ok(MediaOr {
+            span: span!(self, span.lo),
+            condition: media_in_parens,
+        })
+    }
+}
+
+impl<I> Parse<MediaInParens> for Parser<I>
+where
+    I: ParserInput,
+{
+    fn parse(&mut self) -> PResult<MediaInParens> {
+        let state = self.input.state();
+
+        expect!(self, "(");
+
+        self.input.skip_ws()?;
+
+        if !is!(self, "(") && !is!(self, "not") {
+            self.input.reset(&state);
+
+            return Ok(MediaInParens::Feature(self.parse()?));
         }
 
-        Ok(base)
+        let condition = MediaInParens::MediaCondition(self.parse()?);
+
+        expect!(self, ")");
+
+        return Ok(condition);
+    }
+}
+
+impl<I> Parse<MediaFeature> for Parser<I>
+where
+    I: ParserInput,
+{
+    fn parse(&mut self) -> PResult<MediaFeature> {
+        let span = self.input.cur_span()?;
+
+        expect!(self, "(");
+
+        self.input.skip_ws()?;
+
+        let left = self.parse()?;
+
+        self.input.skip_ws()?;
+
+        match cur!(self) {
+            tok!(")") => {
+                eat!(self, ")");
+
+                let name = match left {
+                    MediaFeatureValue::Ident(ident) => MediaFeatureName::Ident(ident),
+                    _ => {
+                        return Err(Error::new(span, ErrorKind::Expected("identifier value")));
+                    }
+                };
+
+                Ok(MediaFeature::Boolean(MediaFeatureBoolean {
+                    span: span!(self, span.lo),
+                    name,
+                }))
+            }
+            tok!(":") => {
+                eat!(self, ":");
+
+                self.input.skip_ws()?;
+
+                let name = match left {
+                    MediaFeatureValue::Ident(ident) => MediaFeatureName::Ident(ident),
+                    _ => {
+                        return Err(Error::new(span, ErrorKind::Expected("identifier value")));
+                    }
+                };
+                let value = self.parse()?;
+
+                self.input.skip_ws()?;
+
+                expect!(self, ")");
+
+                Ok(MediaFeature::Plain(MediaFeaturePlain {
+                    span: span!(self, span.lo),
+                    name,
+                    value,
+                }))
+            }
+            tok!("<") | tok!(">") | tok!("=") => {
+                let left_comparison = match bump!(self) {
+                    tok!("<") => {
+                        if eat!(self, "=") {
+                            MediaFeatureRangeComparison::Le
+                        } else {
+                            MediaFeatureRangeComparison::Lt
+                        }
+                    }
+                    tok!(">") => {
+                        if eat!(self, "=") {
+                            MediaFeatureRangeComparison::Ge
+                        } else {
+                            MediaFeatureRangeComparison::Gt
+                        }
+                    }
+                    tok!("=") => MediaFeatureRangeComparison::Eq,
+                    _ => {
+                        // TODO another error
+                        return Err(Error::new(span, ErrorKind::InvalidCharsetAtRule));
+                    }
+                };
+
+                self.input.skip_ws()?;
+
+                let center = self.parse()?;
+
+                self.input.skip_ws()?;
+
+                if eat!(self, ")") {
+                    return Ok(MediaFeature::Range(MediaFeatureRange {
+                        span: span!(self, span.lo),
+                        left,
+                        comparison: left_comparison,
+                        right: center,
+                    }));
+                }
+
+                let right_comparison = match bump!(self) {
+                    tok!("<") => {
+                        if eat!(self, "=") {
+                            MediaFeatureRangeComparison::Le
+                        } else {
+                            MediaFeatureRangeComparison::Lt
+                        }
+                    }
+                    tok!(">") => {
+                        if eat!(self, "=") {
+                            MediaFeatureRangeComparison::Ge
+                        } else {
+                            MediaFeatureRangeComparison::Gt
+                        }
+                    }
+                    _ => {
+                        // TODO another error
+                        return Err(Error::new(span, ErrorKind::InvalidCharsetAtRule));
+                    }
+                };
+
+                self.input.skip_ws()?;
+
+                let right = self.parse()?;
+
+                self.input.skip_ws()?;
+
+                expect!(self, ")");
+
+                let name = match center {
+                    MediaFeatureValue::Ident(ident) => MediaFeatureName::Ident(ident),
+                    _ => {
+                        return Err(Error::new(span, ErrorKind::Expected("identifier value")));
+                    }
+                };
+
+                // TODO validate comparison
+
+                Ok(MediaFeature::RangeInterval(MediaFeatureRangeInterval {
+                    span: span!(self, span.lo),
+                    left,
+                    left_comparison,
+                    name,
+                    right_comparison,
+                    right,
+                }))
+            }
+            _ => {
+                return Err(Error::new(span, ErrorKind::Expected("identifier value")));
+            }
+        }
+    }
+}
+
+impl<I> Parse<MediaFeatureValue> for Parser<I>
+where
+    I: ParserInput,
+{
+    fn parse(&mut self) -> PResult<MediaFeatureValue> {
+        let span = self.input.cur_span()?;
+
+        match cur!(self) {
+            Token::Num { .. } => {
+                let left = self.parse()?;
+
+                self.input.skip_ws()?;
+
+                if eat!(self, "/") {
+                    self.input.skip_ws()?;
+
+                    let right = self.parse()?;
+
+                    return Ok(MediaFeatureValue::Ratio(BinValue {
+                        span: span!(self, span.lo),
+                        op: BinOp::Div,
+                        left: Box::new(Value::Number(left)),
+                        right: Box::new(Value::Number(right)),
+                    }));
+                }
+
+                return Ok(MediaFeatureValue::Number(left));
+            }
+            Token::Ident { .. } => Ok(MediaFeatureValue::Ident(self.parse()?)),
+            Token::Dimension { .. } => Ok(MediaFeatureValue::Dimension(self.parse()?)),
+            _ => {
+                return Err(Error::new(
+                    span,
+                    ErrorKind::Expected("number, dimension, identifier or ration value"),
+                ));
+            }
+        }
     }
 }
 
@@ -1032,10 +1405,10 @@ where
             }
         };
 
-        return Ok(LayerRule {
+        Ok(LayerRule {
             span: span!(self, span.lo),
             prelude,
             rules,
-        });
+        })
     }
 }
